import argparse
import datetime
import sys
import urllib2
from wpupdater.WPBlog import WPBlog
from wpupdater.SeoChecker import *
from wpupdater.utils import LoginException, WPLoggerAdapter, check_for_new_version

def main():
    ap = argparse.ArgumentParser()
    ap.add_argument('-u', '--username', help='wp-admin login username', required=True)
    ap.add_argument('-p', '--password', help='wp-admin login password', required=True)
    ap.add_argument('-l', '--login-check', dest='login-check', help='Check if credentials are ok', action='store_true', default=False)
    ap.add_argument('-U', '--upgrade', help='Do upgrade blog', action='store_true', default=False)
    ap.add_argument('-r', '--is-upgreadeable', dest='check-if-ready', action='store_true', default=False, help='Check if blog is upgradeable')
    ap.add_argument('url', action='store',help='WordPress address e.g.<www.mywordpress.com>')


    start_time = datetime.datetime.now()
    args = vars( ap.parse_args() )
    blog = WPBlog(args['url'], args['username'], args['password'])
    ret_code = 0
    try:
        blog.login()
        if args['check-if-ready']:
            if blog.check_if_upgradeable():
                print "WordPress site %s is upgradeable!"%args['url']
                ret_code = 0
            else:
                print "WordPress site %s is up-to-date!"%args['url']
                ret_code = 1

        elif args['login-check']:
            """ If no exception returned, credentials are ok """
            print 'Credentials are ok'
            ret_code = 0

        elif args['upgrade']:
            if blog.check_if_upgradeable():
                version = blog.get_current_version()
                print '\t -> \033[92m Current Version = %s, is upgradeable \033[0m'%(version)
                print '\t ->Upgrading....'
                if blog.upgrade():
                    print "\t\t \033[92m -> UPGRADED! \033[0m"
                    ret_code = 0
                else:
                    print "\t\t \033[91m -> UPGRADE ERROR!\033[0m"
                    ret_code = 1

        else:
            print "No args passed, exiting..."
            ret_code = 0

    except LoginException as e:
        print "\t \033[91m -> Login Error! \033[0m"
        ret_code = 1
    except urllib2.URLError:
        print "\t \033[91m -> Error Opening! \033[0m"
        ret_code = 1
    except Exception as e:
        print "UNKNOWN ERROR %s"%e
        ret_code = 1
    stop_time = datetime.datetime.now()
    print "Ended up in %s seconds"%(stop_time - start_time).total_seconds()
    sys.exit(ret_code)

if __name__ == "__main__":
    main()



#
# Part of wpupdater package
#


def check_for_new_version():
    rt_code = False
    CURR_VERSION_FILE = os.path.join(os.path.dirname(os.path.realpath(__file__)), 'WP_CURRENT_VERSION')
    r = urllib2.urlopen('http://wordpress.org/latest.tar.gz.md5')
    remote_md5 = r.read().strip()
    f = open(CURR_VERSION_FILE, 'r')
    local_md5 = f.read().strip()
    f.close()
    if remote_md5 != local_md5:
        rt_code = True
        f = open(CURR_VERSION_FILE, 'w+')
        f.write(remote_md5)
        f.close()
    return rt_code

class LoginException(Exception):
    def __init__(self, msg):
        self.msg = msg


def loggedin(func):
    def f(self, *args, **kwargs):
        if not self.logged_in:
            raise LoginException("Not logged in!")
        return func(self, *args, **kwargs)
    return f



from utils import *
import urllib, urllib2, cookielib
import lxml.html
import re
import logging
import datetime

class WPBlog:

    logged_in = False
    upgrade_page_url = "/wp-admin/update-core.php"

    def __init__(self, domain, user, password):
        self.domain = domain
        self.user = user
        self.password = password
        self.cookiejar = cookielib.CookieJar()
        self.opener = urllib2.build_opener(urllib2.HTTPCookieProcessor(self.cookiejar))
        self.opener.addheaders = [('User-agent', 'Opera/9.80 (X11; Linux x86_64) Presto/2.12.388 Version/12.16')]

    def login(self):
        login_parameters = urllib.urlencode({'log': self.user, 'pwd': self.password})
        self.opener.open("http://www.%s/wp-login.php"%self.domain, login_parameters).close()
        if self.is_looged_in():
            self.logged_in = True
            return True
        raise LoginException('Login Error')

    def search_for_element_id(self, document_to_parse, element_id):
        document = lxml.html.document_fromstring(document_to_parse)
        doc = document.xpath("//@id=\'%s\'"%element_id)
        if doc:
            return True
        return False

    def find_element_xpath(self, document, xpath_exp):
        document = lxml.html.document_fromstring(document)
        element = document.xpath(xpath_exp)
        if element:
            return element
        return False

    def is_looged_in(self):
        response = self.opener.open('http://www.%s/wp-admin/index.php'%self.domain)
        if self.search_for_element_id(response.read(), 'loginform'):
            self.logged_in = False
            return False
        return True

    def open(self, url):
        return self.opener.open('http://www.%s%s'%(self.domain, url))

    def count_links(self, url='/'):
        document = self.open(url).read()
        data = lxml.html.document_fromstring(document)
        return int(len(list(data.iterlinks())))

    @loggedin
    def admin_open(self, url, data = None):
        return self.opener.open('http://www.%s%s'%(self.domain, url), data)

    @loggedin
    def check_if_upgradeable(self):
        upgrade_page = self.admin_open(self.upgrade_page_url).read()
        form = self.find_element_xpath(upgrade_page, "//form[contains(@action, 'do-core-upgrade')]")
        if not form:
            return False
        return True

    def get_nonce_filed(self, form):
        return form.xpath('//input[@name=\"_wpnonce\"]')[0].value

    def get_upgrade_version(self, form):
        return form.xpath('//input[@name=\"version\"]')[0].value

    @loggedin
    def get_plugin_updates_count(self):
        index_page = self.admin_open('/wp-admin/index.php').read()
        plugin_count = self.find_element_xpath(index_page,"//span[@class=\"plugin-count\"]")
        if len(plugin_count) > 0:
            return plugin_count[0].text
        return None

    @loggedin
    def get_current_version(self):
        index_page = self.admin_open('/wp-admin/index.php').read()
        body_class = self.find_element_xpath(index_page, '//body')[0].attrib['class']
        s = re.search('version-([0-9-]*)',body_class)
        if not s:
            return None
        return s.group(1).replace('-','.')

    @loggedin
    def send_upgrade_form(self, wponce, version):
        upgrade_form_params = {
            '_wpnonce':  wponce,
            'version': version,
            'locale': 'pl_PL',
            '_wp_http_referer': '/wp-admin/update-core.php',
            'upgrade': 'upgrade'
        }
        form_params = urllib.urlencode(upgrade_form_params);
        resp = self.admin_open('/wp-admin/update-core.php?action=do-core-upgrade', form_params)
        resp.read()
        if int(resp.getcode()) == 200:
            return True
        return False

    @loggedin
    def upgrade(self, check=True):
        links_before = self.count_links()
        upgrade_page = self.admin_open(self.upgrade_page_url).read()
        form_to_upgrade = self.find_element_xpath(upgrade_page, "//form[contains(@action, 'do-core-upgrade') and @name=\"upgrade\"]")
        if not form_to_upgrade:
            return False
        self.send_upgrade_form(self.get_nonce_filed(form_to_upgrade[0]), self.get_upgrade_version(form_to_upgrade[0]))
        links_after = self.count_links()
        if links_before == links_after or (links_before0) == links_after or (links_before+1) == links_after:
            return True
        return False




try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

config = {
    'description': 'WordPress Updater',
    'author': 'Marcin Matlaszek, inFakt DevTeam',
    'url': 'github.com/infakt/wpupdater',
    'download_url': 'github.com/infakt/wpupdater',
    'author_email': 'marcin.matlaszek@infakt.pl, devteam@infakt.pl',
    'version': '0.1',
    'install_requires': ['lxml'],
    'packages': ['wpupdater'],
    'scripts': ['bin/wpupdate.py'],
    'name': 'wpupdater'
}

setup(**config)
